@@ -22,12 +22,9 @@ import logging
 from typing import (
     TYPE_CHECKING,
     Collection,
-    Dict,
     Iterable,
-    List,
     Mapping,
     Optional,
-    Tuple,
     Union,
 )
 
@@ -143,7 +140,7 @@ class ApplicationServicesHandler:
                         event_to_received_ts.keys(), get_prev_content=True
                     )
 
-                    events_by_room: Dict[str, List[EventBase]] = {}
+                    events_by_room: dict[str, list[EventBase]] = {}
                     for event in events:
                         events_by_room.setdefault(event.room_id, []).append(event)
 
@@ -341,7 +338,7 @@ class ApplicationServicesHandler:
     @wrap_as_background_process("notify_interested_services_ephemeral")
     async def _notify_interested_services_ephemeral(
         self,
-        services: List[ApplicationService],
+        services: list[ApplicationService],
         stream_key: StreamKeyType,
         new_token: Union[int, MultiWriterStreamToken],
         users: Collection[Union[str, UserID]],
@@ -429,7 +426,7 @@ class ApplicationServicesHandler:
 
     async def _handle_typing(
         self, service: ApplicationService, new_token: int
-    ) -> List[JsonMapping]:
+    ) -> list[JsonMapping]:
         """
         Return the typing events since the given stream token that the given application
         service should receive.
@@ -464,7 +461,7 @@ class ApplicationServicesHandler:
 
     async def _handle_receipts(
         self, service: ApplicationService, new_token: MultiWriterStreamToken
-    ) -> List[JsonMapping]:
+    ) -> list[JsonMapping]:
         """
         Return the latest read receipts that the given application service should receive.
 
@@ -503,7 +500,7 @@ class ApplicationServicesHandler:
         service: ApplicationService,
         users: Collection[Union[str, UserID]],
         new_token: Optional[int],
-    ) -> List[JsonMapping]:
+    ) -> list[JsonMapping]:
         """
         Return the latest presence updates that the given application service should receive.
 
@@ -523,7 +520,7 @@ class ApplicationServicesHandler:
             A list of json dictionaries containing data derived from the presence events
             that should be sent to the given application service.
         """
-        events: List[JsonMapping] = []
+        events: list[JsonMapping] = []
         presence_source = self.event_sources.sources.presence
         from_key = await self.store.get_type_stream_id_for_appservice(
             service, "presence"
@@ -563,7 +560,7 @@ class ApplicationServicesHandler:
         service: ApplicationService,
         new_token: int,
         users: Collection[Union[str, UserID]],
-    ) -> List[JsonDict]:
+    ) -> list[JsonDict]:
         """
         Given an application service, determine which events it should receive
         from those between the last-recorded to-device message stream token for this
@@ -585,7 +582,7 @@ class ApplicationServicesHandler:
         )
 
         # Filter out users that this appservice is not interested in
-        users_appservice_is_interested_in: List[str] = []
+        users_appservice_is_interested_in: list[str] = []
         for user in users:
             # FIXME: We should do this farther up the call stack. We currently repeat
             #  this operation in _handle_presence.
@@ -612,7 +609,7 @@ class ApplicationServicesHandler:
         #
         # So we mangle this dict into a flat list of to-device messages with the relevant
         # user ID and device ID embedded inside each message dict.
-        message_payload: List[JsonDict] = []
+        message_payload: list[JsonDict] = []
         for (
             user_id,
             device_id,
@@ -761,8 +758,8 @@ class ApplicationServicesHandler:
         return None
 
     async def query_3pe(
-        self, kind: str, protocol: str, fields: Dict[bytes, List[bytes]]
-    ) -> List[JsonDict]:
+        self, kind: str, protocol: str, fields: dict[bytes, list[bytes]]
+    ) -> list[JsonDict]:
         services = self._get_services_for_3pn(protocol)
 
         results = await make_deferred_yieldable(
@@ -786,9 +783,9 @@ class ApplicationServicesHandler:
 
     async def get_3pe_protocols(
         self, only_protocol: Optional[str] = None
-    ) -> Dict[str, JsonDict]:
+    ) -> dict[str, JsonDict]:
         services = self.store.get_app_services()
-        protocols: Dict[str, List[JsonDict]] = {}
+        protocols: dict[str, list[JsonDict]] = {}
 
         # Collect up all the individual protocol responses out of the ASes
         for s in services:
@@ -804,7 +801,7 @@ class ApplicationServicesHandler:
                 if info is not None:
                     protocols[p].append(info)
 
-        def _merge_instances(infos: List[JsonDict]) -> JsonDict:
+        def _merge_instances(infos: list[JsonDict]) -> JsonDict:
             # Merge the 'instances' lists of multiple results, but just take
             # the other fields from the first as they ought to be identical
             # copy the result so as not to corrupt the cached one
@@ -822,7 +819,7 @@ class ApplicationServicesHandler:
 
     async def _get_services_for_event(
         self, event: EventBase
-    ) -> List[ApplicationService]:
+    ) -> list[ApplicationService]:
         """Retrieve a list of application services interested in this event.
 
         Args:
@@ -842,11 +839,11 @@ class ApplicationServicesHandler:
 
         return interested_list
 
-    def _get_services_for_user(self, user_id: str) -> List[ApplicationService]:
+    def _get_services_for_user(self, user_id: str) -> list[ApplicationService]:
         services = self.store.get_app_services()
         return [s for s in services if (s.is_interested_in_user(user_id))]
 
-    def _get_services_for_3pn(self, protocol: str) -> List[ApplicationService]:
+    def _get_services_for_3pn(self, protocol: str) -> list[ApplicationService]:
         services = self.store.get_app_services()
         return [s for s in services if s.is_interested_in_protocol(protocol)]
 
@@ -872,9 +869,9 @@ class ApplicationServicesHandler:
         return True
 
     async def claim_e2e_one_time_keys(
-        self, query: Iterable[Tuple[str, str, str, int]]
-    ) -> Tuple[
-        Dict[str, Dict[str, Dict[str, JsonDict]]], List[Tuple[str, str, str, int]]
+        self, query: Iterable[tuple[str, str, str, int]]
+    ) -> tuple[
+        dict[str, dict[str, dict[str, JsonDict]]], list[tuple[str, str, str, int]]
     ]:
         """Claim one time keys from application services.
 
@@ -896,7 +893,7 @@ class ApplicationServicesHandler:
         services = self.store.get_app_services()
 
         # Partition the users by appservice.
-        query_by_appservice: Dict[str, List[Tuple[str, str, str, int]]] = {}
+        query_by_appservice: dict[str, list[tuple[str, str, str, int]]] = {}
         missing = []
         for user_id, device, algorithm, count in query:
             if not self.store.get_if_app_services_interested_in_user(user_id):
@@ -929,7 +926,7 @@ class ApplicationServicesHandler:
 
         # Patch together the results -- they are all independent (since they
         # require exclusive control over the users, which is the outermost key).
-        claimed_keys: Dict[str, Dict[str, Dict[str, JsonDict]]] = {}
+        claimed_keys: dict[str, dict[str, dict[str, JsonDict]]] = {}
         for success, result in results:
             if success:
                 claimed_keys.update(result[0])
@@ -938,8 +935,8 @@ class ApplicationServicesHandler:
         return claimed_keys, missing
 
     async def query_keys(
-        self, query: Mapping[str, Optional[List[str]]]
-    ) -> Dict[str, Dict[str, Dict[str, JsonDict]]]:
+        self, query: Mapping[str, Optional[list[str]]]
+    ) -> dict[str, dict[str, dict[str, JsonDict]]]:
         """Query application services for device keys.
 
         Users which are exclusively owned by an application service are queried
@@ -954,7 +951,7 @@ class ApplicationServicesHandler:
         services = self.store.get_app_services()
 
         # Partition the users by appservice.
-        query_by_appservice: Dict[str, Dict[str, List[str]]] = {}
+        query_by_appservice: dict[str, dict[str, list[str]]] = {}
         for user_id, device_ids in query.items():
             if not self.store.get_if_app_services_interested_in_user(user_id):
                 continue
@@ -986,7 +983,7 @@ class ApplicationServicesHandler:
         # Patch together the results -- they are all independent (since they
         # require exclusive control over the users). They get returned as a single
         # dictionary.
-        key_queries: Dict[str, Dict[str, Dict[str, JsonDict]]] = {}
+        key_queries: dict[str, dict[str, dict[str, JsonDict]]] = {}
         for success, result in results:
             if success:
                 key_queries.update(result)
